package de.hawhamburg.reachability.reasoner;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.logging.Level;
import java.util.logging.Logger;

import de.hawhamburg.lifecycle.agent.LifeCycleAgent;
import de.hawhamburg.lifecycle.agent.LifeCycleApp;
import de.hawhamburg.lifecycle.data.JsonInfoMessage;
import de.hawhamburg.lifecycle.data.ProjectStatus;
import de.hawhamburg.reachability.ProjectInfo;
import de.hawhamburg.reachability.util.OutputUtil;

/**
 * Class to manage the "Living Place Agent Life Cycle" for {@link Reasoner}s.<br><br>
 * 
 * This class handles the requests related to the "living place agent life cycle" specification, which will be supplied
 * on a global control topic via the ActiveMQ. It implements the {@link LifeCycleAgent} interface and will be 
 * instantiated with a {@link Reasoner}, which will be started immediately. Furthermore it will wrap the running 
 * {@link Reasoner} by a {@link LifeCycleApp}. Nevertheless, this class implements the control actions related to a
 * {@link Reasoner}, such as: "start", "sleep", "info" and "stop".
 * 
 * @author  {@value de.hawhamburg.reachability.ProjectInfo#REACHABILITY_AGENT_AUTHOR}
 * @version {@value de.hawhamburg.reachability.ProjectInfo#REACHABILITY_AGENT_VERSION}
 */
public class ReasonerAgent implements LifeCycleAgent {
    
    /**
     * The current status of the reasoner agent.
     */
    private ProjectStatus myStatus;
    
    /**
     * Time to wait to check again if the worker has already been stopped, while {@link #stop()} is ending the worker 
     * thread, before sending success.  
     */
    private static final long SHUTDOWN_CHECKTIME    = 100;
    
    /**
     * The {@link Reasoner}, a thread which provides reads the internal data format and estimates the current 
     * reachability of the inhabitant.
     */
    private Reasoner worker                         = null;
    
    /**
     * The simple class name of the {@link #worker}.
     */
    private String workerName                       = null;

    /**
     * Create a new {@link ReasonerAgent} which starts the supplied {@link Reasoner} as {@link LifeCycleApp} and
     * manages control messages for this {@link Reasoner}, such as "start", "stop", "info" and "sleep".
     * 
     * @param reasoner  The {@link Reasoner} which shall be started and managed as {@link LifeCycleApp}.    
     */
    public ReasonerAgent(final Reasoner reasoner) {
        this.worker     = reasoner;
        this.workerName = this.worker.getClass().getSimpleName();
        
        OutputUtil.info("*******************************");
        OutputUtil.info("** New life cycle agent '" + this.getClass().getSimpleName() + "' started");
        OutputUtil.info("*******************************");
        OutputUtil.info("** - ID:            '" + this.getID() + "'");
        OutputUtil.info("** - Current state: '" + ProjectStatus.SLEEPING + "'");
        OutputUtil.info("*******************************");
        
        this.worker.start();
        
        LifeCycleApp lifecycle = new LifeCycleApp();
        lifecycle.setLifeCycleAgent(this);
        new Thread(lifecycle).start();
        
        this.myStatus = ProjectStatus.SLEEPING;
    }
    
    /**
     * @see de.hawhamburg.lifecycle.agent.LifeCycleAgent#sleep()
     */
    public final void sleep() {
        OutputUtil.info("Getting Sleeping Order");
        this.myStatus = ProjectStatus.SLEEPING;
        
        this.worker.setSilent(true);
    }

    /**
     * @see de.hawhamburg.lifecycle.agent.LifeCycleAgent#getInfoMessage()
     * 
     * @return          A {@link JsonInfoMessage} object describing the current systems state.
     */
    public final JsonInfoMessage getInfoMessage() {
        JsonInfoMessage msg = new JsonInfoMessage();

        msg.author  = ProjectInfo.REACHABILITY_AGENT_AUTHOR;
        msg.Id      = this.getID();
        msg.status  = this.myStatus.toString();
        msg.Version = ProjectInfo.REACHABILITY_AGENT_VERSION;
        try {
            msg.ip = InetAddress.getLocalHost().toString();
        } catch (UnknownHostException e) {
            Logger.getLogger(ReasonerAgent.class.getName()).log(Level.SEVERE, null, e);
            msg.ip = "unknown";
        }
        msg.misc    = ProjectInfo.REACHABILITY_AGENT_DESCRIPTION;
        
        OutputUtil.info("Sending INFO Message");
        
        return msg;
    }

    /**
     * @see de.hawhamburg.lifecycle.agent.LifeCycleAgent#start()
     */
    public final void start() {
        OutputUtil.info("Getting Starting Order!");
        this.myStatus = ProjectStatus.RUNNING;
        
        this.worker.setSilent(false);
    }

    /**
     * @see de.hawhamburg.lifecycle.agent.LifeCycleAgent#stop()
     */
    public final void stop() {
        OutputUtil.info("Getting Stopping Order!");
        this.myStatus = ProjectStatus.STOPPED;
        
        if (this.worker != null) {
            this.worker.shutDown();
            while (this.worker != null && !this.worker.stopped()) {
                try {
                    Thread.sleep(SHUTDOWN_CHECKTIME);
                } catch (InterruptedException e) {
                    System.err.println(this.getClass().getName() + ": Unable to sleep this thread. " + e.getMessage());
                }
            }
            this.worker = null;
        }

        OutputUtil.info("Worker has stopped!");
    }

    /**
     * @see de.hawhamburg.lifecycle.agent.LifeCycleAgent#getID()
     * 
     * @return          The ID of this service as {@link String} defined by {@link ProjectInfo#REACHABILITY_AGENT_NAME}.
     */
    public final String getID() {
        return ProjectInfo.REACHABILITY_AGENT_NAME + "_Reasoner_"  + this.workerName;
    }
}
