package de.hawhamburg.reachability.reasoner;

import java.util.Map;

import de.hawhamburg.reachability.StopableObject;
import de.hawhamburg.reachability.enumeration.ReachabilityGroup;
import de.hawhamburg.reachability.enumeration.ReachabilityStatus;

/**
 * Default interface for reasoner which needs to be implemented by each reasoning engine.<br><br> 
 * 
 * This interface defines the methods which a reasoner needs to implement to be used in common with this reachability 
 * agent. Besides the specified methods in this interface, the reasoner needs to implement the interfaces
 * {@link Runnable} and {@link StopableObject} as well.
 * 
 * @author  {@value de.hawhamburg.reachability.ProjectInfo#REACHABILITY_AGENT_AUTHOR} 
 * @version {@value de.hawhamburg.reachability.ProjectInfo#REACHABILITY_AGENT_VERSION}
 */
public interface ReasoningEngine extends Runnable,
                                         StopableObject {

    /**
     * The property key which leads to the path where the reasoning engine properties are stored.
     */
    String PROPERTY_FILE_SOURCE_KEY = "REASONING_ENGINE_PROPERTY_SOURCE";
    
    /**
     * Returns the current estimated state of reachability for a certain group from the reasoner.
     * 
     * @param group     A {@link ReachabilityGroup} for which the state of reachability is requested.
     * @return          The current estimated state of reachability as {@link ReachabilityStatus}.
     */
    ReachabilityStatus getEstimatedReachability(ReachabilityGroup group);
    
    /**
     * Returns the current estimated state of reachability of all registered groups as hash map.
     * 
     * @return          A hash map containing each {@link ReachabilityGroup} and its currently estimated state of 
     *                  reachability as {@link ReachabilityStatus}.
     */
    Map<ReachabilityGroup, ReachabilityStatus> getEstimatedReachability();
    
    /**
     * Handles the user feedback from a user interface supplied via the ActiveMQ for a certain {@link ReachabilityGroup}
     * and the users estimation of his {@link ReachabilityStatus} related to this group.
     * 
     * @param group     A {@link ReachabilityGroup} for which the feedback is meant.
     * @param status    The {@link ReachabilityStatus} as users estimation related to the specified group.
     */
    void handleFeedback(ReachabilityGroup group,
                        ReachabilityStatus status);
    
    /**
     * If this method is called, the reasoner dumps its current internal state using the 
     * {@link de.hawhamburg.reachability.util.OutputUtil#info(String)} method.
     */
    void dump();
    
    /**
     * This method of {@link Thread} has been added to the interface to make it public.
     */
    void start();
}
