package de.hawhamburg.reachability.tools.googleCalendar;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;

import com.google.gdata.data.DateTime;
import com.google.gdata.data.calendar.CalendarEventEntry;
import com.google.gson.Gson;

import de.hawhamburg.livingplace.messaging.activemq.wrapper.LPPublisher;
import de.hawhamburg.livingplace.messaging.activemq.wrapper.LPSubscriber;
import de.hawhamburg.reachability.StopableObject;
import de.hawhamburg.reachability.tools.googleCalendar.json.JsonGoogleCalendarEvent;
import de.hawhamburg.reachability.tools.googleCalendar.json.JsonGoogleCalendarRequest;
import de.hawhamburg.reachability.tools.googleCalendar.json.JsonGoogleCalendarResult;
import de.hawhamburg.reachability.util.ConnectionUtil;
import de.hawhamburg.reachability.util.OutputUtil;
import de.hawhamburg.reachability.util.ParameterUtil;

/**
 * Google calendar hook service which receives requests and returns results via the ActiveMQ.<br><br>
 * 
 * This service connects to a google calendar web-service using the {@link GoogleCalendarHook} and awaits external 
 * requests via the ActiveMQ. If a request arises, it will search for fitting results and return them via the ActiveMQ
 * again. This service is designed as stand-alone application.
 *
 * @author  {@value de.hawhamburg.reachability.ProjectInfo#REACHABILITY_AGENT_AUTHOR} 
 * @version {@value de.hawhamburg.reachability.ProjectInfo#REACHABILITY_AGENT_VERSION}
 */
public final class GoogleCalendarHookService extends    Thread
                                             implements StopableObject {
    
    /**
     * The name of the ActiveMQ topic as {@link String} on which the service expects requests.
     */
    private static final String GCS_REQUEST_TOPIC   = "GoogleCalendarRequestTopic";
    
    /**
     * The name of the ActiveMQ topic as {@link String} on which the service publishes its results.
     */
    private static final String GCS_RESULT_TOPIC    = "GoogleCalendarResultTopic";
    
    /**
     * The (fixed...) name of the google user.<br>
     * <b>This needs to be exported to an external file or the message communication during further works.</b>
     */
    private static final String GOOGLE_USER         = "ute.mueller@trashmail.de";
    
    /**
     * The (fixed...) password of the google user.<br>
     * <b>This needs to be exported to an external file or the message communication during further works.</b>
     */
    private static final String GOOGLE_PASSWORD     = "UteMueller";
    
    /**
     * The name of the google calendar which shall be used.
     */
    private static final String GOOGLE_CALENDAR     = "ute.mueller@trashmail.de";
    
    /**
     * The {@link LPSubscriber}, which reads requests from the ActiveMQ.
     */
    private LPSubscriber requestTopic               = null;
    
    /**
     * The {@link LPPublisher}, which writes request messags to the ActiveMQ.
     */
    private LPPublisher resultTopic                 = null;
    
    /**
     * The connection to the google calendar web-service as {@link GoogleCalendarHook}.
     */
    private GoogleCalendarHook calendar             = null;
        
    /**
     * Determines if the method {@link #shutDown()} has been called.
     */
    private boolean shutDown                        = false;
    
    /**
     * Private constructor method to prevent external created instances of this class. Instances will only be created
     * by the {@link #main(String[])} method.
     */
    private GoogleCalendarHookService() { }

    /**
     * @see de.hawhamburg.reachability.StopableObject#isActive()
     * 
     * @return                  <code>true</code> if {@link #shutDown()} wasn't called, otherwise
     *                          <code>false</code>.
     */
    public boolean isActive() {
        return !this.shutDown;
    }

    /**
     * @see de.hawhamburg.reachability.StopableObject#shutDown()
     */
    public void shutDown() {
        this.shutDown = true;
    }
    
    /**
     * Runs the google calendar service. It opens the ActiveMQ connections and the {@link GoogleCalendarHook}. It 
     * waits for requests and sends the results via the ActiveMQ if one arises. After the {@link #shutDown()} method
     * has been called, it gracefully ends this service.
     */
    public void run() {
        init();
        
        List<String> enabledCalendars = calendar.getCalendars();
        if (enabledCalendars.contains(GOOGLE_CALENDAR)) {
            while (!this.shutDown && calendar.isConnected()) {
                handleRequest();
            }
        } else {
            OutputUtil.error("Unable to access the requested calendar '" + GOOGLE_CALENDAR + "'");
        }
        
        end();
    }
    
    /**
     * Initializes connections and class field structures.
     */
    private void init() {
        // Connect to calendar
        calendar = new GoogleCalendarHook(GOOGLE_USER, GOOGLE_PASSWORD);
        OutputUtil.info("GoogleCalendarHookService has been started");
        
        // Open connections
        requestTopic = ConnectionUtil.getSubscriber(GCS_REQUEST_TOPIC, this);
        resultTopic  = ConnectionUtil.getPublisher(GCS_RESULT_TOPIC, this);
    }
    
    /**
     * Gracefully shutdown all open connections.
     */
    private void end() {
        // Reset calendar
        calendar = null;
        OutputUtil.info("GoogleCalendarHookService has been ended by request");
        
        // Close connections if they are still open
        if (requestTopic != null) {
            requestTopic.disconnect();
        }
        if (resultTopic != null) {
            resultTopic.disconnect();
        }
    }
    
    /**
     * The methods handles arising requests via the ActiveMQ and forwards them to the {@link GoogleCalendarHook} by 
     * using {@link #getAndPushCalendarEvents(JsonGoogleCalendarRequest)}.
     */
    private void handleRequest() {
        String json = ConnectionUtil.getMessageOnTopic(requestTopic);
        if (json != null) {
            Gson g = new Gson();
            JsonGoogleCalendarRequest msgObj = g.fromJson(json, JsonGoogleCalendarRequest.class);
            OutputUtil.debug("Calendar events request: " + json);
            getAndPushCalendarEvents(msgObj);
        }
    }
    
    /**
     * This method forwords a request to the {@link GoogleCalendarHook} and publishes its results via the ActiveMQ.
     * 
     * @param request               The users request as {@link JsonGoogleCalendarRequest} message object.
     */
    private void getAndPushCalendarEvents(final JsonGoogleCalendarRequest request) {
        List<CalendarEventEntry> entries = new ArrayList<CalendarEventEntry>();

        DateTime now = new DateTime(System.currentTimeMillis());
        switch (request.requestType) {
            case CURRENT:
                DateTime slightlyNow = new DateTime(now.getValue() + 1);
                entries = calendar.getEvents(GOOGLE_CALENDAR, 1, now, slightlyNow, null);
                break;
            case LAST:
                entries = calendar.getEvents(GOOGLE_CALENDAR, request.amount,
                                             null, now, "descending");
                break;
            case NEXT:
                entries = calendar.getEvents(GOOGLE_CALENDAR, request.amount,
                                             now, null, null);
                break;
            case RANGE:
                entries = calendar.getEvents(GOOGLE_CALENDAR, request.amount,
                                             request.min, request.max, null);
                break;
            case CONTROL_KILL_SENSOR:
                this.shutDown();
                return;
            default:
                OutputUtil.error("Unsupported request type '" + request.requestType + "' supplied");
        }
        
        JsonGoogleCalendarResult resObj = createResultMessage(request, entries);
        OutputUtil.debug("Calendar events result: " + new Gson().toJson(resObj));
        ConnectionUtil.setMessageOnTopic(resultTopic, resObj);
    }
    
    /**
     * Creates a new {@link JsonGoogleCalendarResult} message object which contains the results of the request forwarded
     * to the {@link GoogleCalendarHook}.
     * 
     * @param request               The original request message as {@link JsonGoogleCalendarRequest}.
     * @param entries               The search results of the {@link GoogleCalendarHook} as {@link List} of 
     *                              {@link CalendarEventEntry}.
     * @return                      A new proper {@link JsonGoogleCalendarResult} containing the request results.
     */
    private JsonGoogleCalendarResult createResultMessage(final JsonGoogleCalendarRequest request,
                                                         final List<CalendarEventEntry> entries) {
        JsonGoogleCalendarResult msg = new JsonGoogleCalendarResult();
        msg.requestId   = request.Id;
        msg.requestType = request.requestType;
        msg.amount      = entries.size();
        
        List<JsonGoogleCalendarEvent> events = new ArrayList<JsonGoogleCalendarEvent>();
        for (CalendarEventEntry e : entries) {
            JsonGoogleCalendarEvent event = new JsonGoogleCalendarEvent();
            event.name          = e.getTitle().getPlainText();
            event.description   = e.getPlainTextContent();
            if (e.getLocations().size() > 0) {
                event.location      = e.getLocations().get(0).getValueString();
            }
            event.httpLink      = e.getHtmlLink().getHref();
            event.startTime     = e.getTimes().get(0).getStartTime();
            event.endTime       = e.getTimes().get(0).getEndTime();
            events.add(event);
        }
        msg.events = events;
        
        return msg;
    }
    
    /**
     * This method starts a new {@link GoogleCalendarHookService}.
     * 
     * @param args              The command line arguments as array of {@link String}.
     */
    public static void main(final String[] args) {
        // Primary handling of parameters
        String[] subset = {"--wrapped"};
        ParameterUtil p = ParameterUtil.fromArgs(args, subset);
        
        if (!p.isset("--wrapped")) {
            // Remove all existing loggers
            Logger.getRootLogger().removeAllAppenders();
        }
        
        // Start calendar hook
        new GoogleCalendarHookService().start();
    }
    
}
